package cr311h2;

/**
 * Copyright 2005, DocJava, Inc.
 * User: dlyon
 * Date: Jan 23, 2006
 * Time: 4:11:12 PM
 */

import futils.Futil;
import futils.ListFiles;
import gui.ClosableJFrame;
import gui.run.RunButton;
import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import j2d.ImageUtils;
import j2d.Images;
import j2d.gui.ImagePanel;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.MissingResourceException;
import java.util.StringTokenizer;
import java.util.Vector;

//SplitPaneDemo itself is not a visible component.

public class ImageBrowser implements ListSelectionListener {
    private Vector imageFileVector;
    private JLabel picture;
    private JList jListOfImageNames;
    private JSplitPane splitPane;

    public ImageBrowser() {

        //Read image names from a properties file
        //todo for hw1 you must remove the resource bundle and use
        // the jListOfImageNames of images from the getFileList to run your
        // program.

        try {
            imageFileVector = getImageFiles();
        } catch (MissingResourceException e) {
            System.out.println("Can't find the properties file " +
                    "that contains the image names.");
            return;
        }

        //Create the jListOfImageNames of images and put it in a scroll pane
        jListOfImageNames = getJList();
        JScrollPane listScrollPane = new JScrollPane(jListOfImageNames);

        //Set up the picture label and put it in a scroll pane
        ImageIcon firstImage = new ImageIcon(imageFileVector.firstElement().toString());
        picture = new JLabel(firstImage);
        picture.setPreferredSize(new Dimension(firstImage.getIconWidth(),
                firstImage.getIconHeight()));
        JScrollPane pictureScrollPane = new JScrollPane(picture);

        //Create a split pane with the two scroll panes in it
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                listScrollPane, pictureScrollPane);
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(150);

        //Provide minimum sizes for the two components in the split pane
        Dimension minimumSize = new Dimension(100, 50);
        listScrollPane.setMinimumSize(minimumSize);
        pictureScrollPane.setMinimumSize(minimumSize);

        //Provide a preferred size for the split pane
        splitPane.setPreferredSize(new Dimension(400, 200));
    }

    private JList getJList() {
        JList jl = new JList(getFileNames(imageFileVector));
        jl.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jl.setSelectedIndex(0);
        jl.addListSelectionListener(this);
        return jl;
    }

    private Vector getFileNames(Vector imageNames) {
        Vector v = new Vector();
        for (int i = 0; i < imageNames.size(); i++) {
            File f = (File) imageNames.elementAt(i);
            v.addElement(f.getName());
        }
        return v;
    }


    //todo get this working
    private Vector getImageFiles() {
        File f[] = ListFiles.getFileList("select an image directory");
        Vector v = new Vector();
        for (int i = 0; i < f.length; i++)
            v.addElement(f[i]);
        return v;
    }

    //Used by SplitPaneDemo2
    public JList getImageList() {
        return jListOfImageNames;
    }

    public JSplitPane getSplitPane() {
        return splitPane;
    }

    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting())
            return;

        JList theList = (JList) e.getSource();
        if (theList.isSelectionEmpty()) {
            picture.setIcon(null);
        } else {
            int index = theList.getSelectedIndex();
            String filename = imageFileVector.elementAt(index).toString();
            ImageIcon newImage = new ImageIcon(filename);
            picture.setIcon(newImage);
            picture.setPreferredSize(new Dimension(newImage.getIconWidth(),
                    newImage.getIconHeight()));
            picture.revalidate();
        }
    }

    protected static Vector parseList(String theStringList) {
        Vector v = new Vector(10);
        StringTokenizer tokenizer = new StringTokenizer(theStringList, " ");
        while (tokenizer.hasMoreTokens()) {
            String image = tokenizer.nextToken();
            v.addElement(image);
        }
        return v;
    }

    public static void main(String s[]) {
        ImageBrowser ib = new ImageBrowser();
        ib.start();
    }

    public void addMenu(JFrame jf) {
        JMenuBar jm = jf.getJMenuBar();
    }

    public JMenuBar getMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        return mb;
    }

    public RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");
        fileMenu.add(new RunMenuItem("[close{alt shift Y}") {
            public void run() {
                System.out.println(getText());
            }
        });
        fileMenu.add(new RunMenuItem("[open{alt shift X}") {
            public void run() {
                System.out.println(Futil.getReadFileDir("select a directory"));
            }
        });
        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            }
        });
        fileMenu.add(new RunMenuItem("append images") {
            public void run() {
                appendImages();
            }


        });
        fileMenu.add(new RunMenuItem("[save{alt control DELETE}") {
            public void run() {
                System.out.println(getText());
            }
        });
        return fileMenu;
    }

    //todo implement this for HW2
    private void appendImages() {
        Images imgs = getImages();
        Image img[] = imgs.getImages();
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        //todo try the ImagePanel
        for (int i = 0; i < img.length; i++) {
            ImagePanel ip = new ImagePanel();
            ip.setImage(ImageUtils.scaleFast(img[i],100,100));
            c.add(ip);
        }
        c.setLayout(new GridLayout(0,5));
        cf.setSize(200, 200);
        cf.setVisible(true);
        //Dimension d = new Dimension(100,100);
        //Image image = imgs.getImage(d,6,8);
        //ImageUtils.displayImage(image,"compositeImage");

    }

    public Images getImages() {
        Images imgs = new Images();
        for (int i = 0; i < imageFileVector.size(); i++) {
            File x = (File) imageFileVector.elementAt(i);
            System.out.println(x);
            if (x.isFile()) {
                Image image = ImageUtils.getImage(x);
                if (image != null) imgs.add(image);
            }
        }
        return imgs;
    }

    public void start() {
        JFrame frame = new JFrame("Image Browser");
        frame.setJMenuBar(getMenuBar());
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        ImageBrowser splitPaneDemo = new ImageBrowser();
        frame.getContentPane().add(splitPaneDemo.getSplitPane());
        frame.pack();
        frame.setVisible(true);
    }
}


